/*
 * fconcom.c	F-CON communication and utilities
 *			Copyright(C)2003 Kashi Electronics Designs,
 *			All Rights Reserved.
 *	Ver 1.0	Sep.8/2003 first release
 *
 *   This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <pic18.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fcon.h"
#include "config.h"
#include "keytimer.h"

#define DIVIDER		((int)(FOSC/(16UL * BAUDRATE) -1))

unsigned char		s_buf[24], r_buf[24];		// transeive buffer
unsigned char		mon_buf[128];				// monitor packet storage
volatile char		g_commerr;
unsigned char		g_fcontype;				// FCON model in connection

short			fcon_GetMonValW( unsigned char );
unsigned char		fcon_GetMonVal( unsigned char );


void	init_comm(void)
{
	SPBRG = DIVIDER;
	TXSTA = 0x24;
	RCSTA = 0x90;
	TRISC6 = 0; TRISC7 = 1;
	g_commerr = 0;
}

void	s_putch(unsigned char byte)
{
	/* output one byte */
	while(!TXIF)	/* set when register is empty */
		continue;
	TXREG = byte;
}

unsigned char	s_getch()
{
	/* retrieve one byte */
	g_commto = 4;				// timeout 100ms
	while(!RCIF){				/* set when register is not empty */
		if( !g_commto ){
			g_commerr = 1;
			break;
		}
	}

	return RCREG;	
}

void		s_rcvflash()
{
	char		i, j;

	for( i = 0; i < 24; i++){
		g_commto = 16;			// timeout 400ms
		while(!RCIF){
			if( !g_commto )
				return;
		}
		j = RCREG;				// dummy read
	}
}

void	fcon_read16( unsigned short addr, unsigned char sec)
{
	char	i, sum;

	switch(g_fcontype){
	case FCT_VP31:
		s_buf[0] = 0x3c;
		s_buf[3] = 0;	s_buf[4] = sec;
		s_buf[5] = *((char *)&addr + 1);	// msb
		s_buf[6] = *((char *)&addr);		// lsb
		break;

	case FCT_S:
		s_buf[0] = 0xa5;
		s_buf[4] = 0;	s_buf[3] = sec;
		s_buf[5] = *((char *)&addr);		// lsb
		s_buf[6] = *((char *)&addr + 1);	// msb
		break;

	case FCT_VP30:
		s_buf[0] = 0xa5;
		s_buf[3] = 0;	s_buf[4] = sec;
		s_buf[5] = *((char *)&addr + 1);	// msb
		s_buf[6] = *((char *)&addr);		// lsb
		break;

	case FCT_VP2:
		s_buf[0] = 0x5a;
		s_buf[4] = 0;	s_buf[3] = sec;
		s_buf[5] = *((char *)&addr);		// lsb
		s_buf[6] = *((char *)&addr + 1);	// msb
		break;
	}

	s_buf[1] = 0x3a; s_buf[2] = 0x10;
	for(i = 7; i < 24; i++)
		s_buf[i] = 0;

	sum = 0;
	s_putch(s_buf[0]);
	for(i = 1; i < 23; i++){
		s_putch(s_buf[i]);
		sum += s_buf[i];
	}
	s_putch((unsigned char)-sum);

	for(i = 0; i < 24; i++){
		r_buf[i] = s_getch();
		if(g_commerr){
			s_rcvflash();
			return;
		}
	}
}

void	fcon_getmon16(unsigned char a)
{
	unsigned short	addr;
	unsigned char		i, j;

	j = a & 0xf0;
	switch(g_fcontype){
	case FCT_VP31:
		addr = j + 0x4000;
		i = 0x80;
		break;
	case FCT_S:
		addr = j + 0xef00;
		i = 0xff;
		break;
	case FCT_VP30:
		addr = j + 0x4000;
		i = 0x80;
		break;
	case FCT_VP2:
		addr = j + 0xfc00;
		i = 0;
		break;
	}
	fcon_read16( addr, i);
	for(i = 0; i < 16; i++)
		mon_buf[i + j] = r_buf[i + 7];
}

short	fcon_GetMonValW( unsigned char addr)
{
	short	v;

	if( g_fcontype == FCT_VP2 ){				// big endian VP(silver)
		*((char *)&v) = mon_buf[addr];
		*((char *)&v+1) = mon_buf[addr + 1];

	} else {
		*((char *)&v) = mon_buf[addr + 1];		// little endian others
		*((char *)&v+1) = mon_buf[addr];
	}

	return	v;
}

unsigned char	fcon_GetMonVal( unsigned char addr)
{
	return mon_buf[addr];
}


void		int2str( char *buf, short val )
{
	sprintf( buf, "%4d", val);
}


void		uint2str( char *buf, unsigned short val )
{
	sprintf( buf, "%4u", val);
}


void		im52str( char *buf, short val )
{
	sprintf( buf, "%4d", val - 50);
}


void		char2str( char *buf, unsigned char val )
{
	sprintf( buf, "%3d", val);
}

void		nchar2str( char *buf, signed char val )
{
	sprintf( buf, "%3d", -val);
}


void		deg2str( char *buf, unsigned short val )
{
	sprintf( buf, "%2d.%1d", (val / 10), (val % 10));
}


void		c052str( char *buf, unsigned char val )
{
	unsigned short	v;

	v = val * 5;
	sprintf( buf, "%2d.%1d", (v / 10), (v % 10));
}


void		cent2str( char *buf, unsigned short val )
{
	sprintf( buf, "%2d.%02d", (val / 100), (val % 100));
}


void		i1k2str( char *buf, unsigned short val )
{
	sprintf( buf, "%2d.%02d", (val / 1000), ((val % 1000) / 10));
}


void		i352str( char *buf, unsigned short val )
{
	long		v;
	short	i, j;
	v = val * 13671L;
	i = v / 100000L - 100L;
	j = abs(i);
	sprintf( buf, "%c%d.%02d", i < 0 ? '-':' ', (j / 100), (j % 100));

}


void		bool2str( char *buf, unsigned char val )
{
	if( val ){
		strcpy( buf, "ON");
	} else {
		strcpy( buf, "OFF");
	}
}

void		stralign( char *buf, char width)
{
	char		i;

	i = strlen(buf);
	if( i >= width ) return;
	buf[width] = 0;
	while(i){
		width--; i--;
		buf[width] = buf[i];
	}
	while(width){
		width--;
		buf[width] = 0x20;
	}
}
